@@ -16,14 +16,15 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
   if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "all"))  mult = ALL;
   else if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "first")) mult = FIRST;
   else if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "last")) mult = LAST;
-  else error("Internal error: invalid value for 'mult'; this should have been caught before. please report to data.table issue tracker");
+  else error("Internal error: invalid value for 'mult'; this should have been caught before. please report to data.table issue tracker"); // # nocov
 
   if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "any"))  type = ANY;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "within")) type = WITHIN;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "start")) type = START;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "end")) type = END;
-  else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "equal")) type = EQUAL;
-  else error("Internal error: invalid value for 'type'; this should have been caught before. please report to data.table issue tracker");
+  // TODO : re-cover when implemented
+  else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "equal")) type = EQUAL; // # nocov
+  else error("Internal error: invalid value for 'type'; this should have been caught before. please report to data.table issue tracker"); // # nocov
 
   // For reference: uxcols-1 = type_count, uxcols-2 = count, uxcols-3 = type_lookup, uxcols-4 = lookup
   // first pass: calculate lengths first
@@ -76,12 +77,15 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
         }
       }
       break;
+    // TODO : re-cover when implemented
+    // # nocov start
     case EQUAL:
       for (i=0; i<xrows; i++) {
         len1[from[i]-1]++; len1[to[i]-1]++;
         len2[from[i]-1]++; len2[to[i]-1]++;
       }
       break;
+    // # nocov end
     case ANY :
       for (i=0; i<xrows; i++) {
         k = from[i];
@@ -129,12 +133,15 @@ SEXP lookup(SEXP ux, SEXP xlen, SEXP indices, SEXP gaps, SEXP overlaps, SEXP mul
       }
     }
     break;
+  // TODO : re-cover when implemented
+  // # nocov start
   case EQUAL:
     for (i=0; i<xrows; i++) {
       INTEGER(VECTOR_ELT(lookup, from[i]-1))[idx[from[i]-1]++] = i+1;
       INTEGER(VECTOR_ELT(lookup, to[i]-1))[idx[to[i]-1]++] = i+1;
     }
     break;
+  // # nocov end
   }
   Free(idx);
   // generate type_lookup
@@ -214,14 +221,14 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
   if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "all"))  mult = ALL;
   else if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "first")) mult = FIRST;
   else if (!strcmp(CHAR(STRING_ELT(multArg, 0)), "last")) mult = LAST;
-  else error("Internal error: invalid value for 'mult'; this should have been caught before. please report to data.table issue tracker");
+  else error("Internal error: invalid value for 'mult'; this should have been caught before. please report to data.table issue tracker"); // # nocov
 
   if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "any"))  type = ANY;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "within")) type = WITHIN;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "start")) type = START;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "end")) type = END;
   else if (!strcmp(CHAR(STRING_ELT(typeArg, 0)), "equal")) type = EQUAL;
-  else error("Internal error: invalid value for 'type'; this should have been caught before. please report to data.table issue tracker");
+  else error("Internal error: invalid value for 'type'; this should have been caught before. please report to data.table issue tracker"); // # nocov
 
   // As a first pass get the final length, so that we can allocate up-front and not deal with Calloc + Realloc + size calculation hassle
   // Checked the time for this loop on realisitc data (81m reads) and took 0.27 seconds! No excuses ;).
@@ -234,6 +241,8 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
         totlen += (from[i] > 0 && len2[from[i]-1]) ? len2[from[i]-1] : 1;
       break;
 
+    // TODO : re-cover when implemented
+    // # nocov start
     case EQUAL:
       for (i=0; i<rows; i++) {
         len = totlen; wlen=0, j=0, m=0;
@@ -256,6 +265,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
           ++totlen;
       }
       break;
+    // # nocov end
 
     case ANY:
       for (i=0; i<rows; i++) {
@@ -334,6 +344,8 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
+    // TODO : re-cover when implemented
+    // # nocov start
     case EQUAL :
       for (i=0; i<rows; i++) {
         len = thislen;
@@ -369,6 +381,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
+    // # nocov end
 
     case ANY :
       for (i=0; i<rows; i++) {
@@ -456,6 +469,8 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
+    // TODO : re-cover when implemented
+    // # nocov start
     case EQUAL :
       for (i=0; i<rows; i++) {
         len = thislen;
@@ -487,6 +502,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
+    // # nocov end
 
     case ANY:
       for (i=0; i<rows; i++) {
@@ -562,6 +578,8 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
       }
       break;
 
+    // TODO : re-cover when implemented
+    // # nocov start
     case EQUAL :
       for (i=0; i<rows; i++) {
         len = thislen;
@@ -593,6 +611,7 @@ SEXP overlaps(SEXP ux, SEXP imatches, SEXP multArg, SEXP typeArg, SEXP nomatchAr
          }
       }
       break;
+    // # nocov end
 
       // OLD logic for 'any,last' which had to check for maximum for each 'i'. Better logic below.
       // for 'first' we need to just get the minimum of first non-zero-length element, but not the same case for 'last'.
